import {
  One,
  and,
  asc,
  desc,
  eq,
  getTableColumns,
  gt,
  gte,
  ilike,
  inArray,
  is,
  isNotNull,
  isNull,
  like,
  lt,
  lte,
  ne,
  notIlike,
  notInArray,
  notLike,
  or
} from "drizzle-orm";
import {
  GraphQLBoolean,
  GraphQLEnumType,
  GraphQLError,
  GraphQLInputObjectType,
  GraphQLInt,
  GraphQLList,
  GraphQLNonNull,
  GraphQLObjectType,
  GraphQLString,
  Kind
} from "graphql";
import { parseResolveInfo } from "graphql-parse-resolve-info";
import { drizzleColumnToGraphQLType } from "../../TypeConverter/vanilla";
import { pascalize } from "../../caseOps";
import { remapFromGraphQLCore } from "../../dataMappers";
const extractSelectedColumns = (info, queryName) => {
  const tableSelection = info.operation.selectionSet.selections.find(
    (e) => e.kind === Kind.FIELD && e.name.value === queryName
  );
  if (!tableSelection || !tableSelection.selectionSet)
    return {};
  const selectedColumns = [];
  for (const columnSelection of tableSelection.selectionSet.selections) {
    if (columnSelection.kind !== Kind.FIELD)
      continue;
    selectedColumns.push([columnSelection.name.value, true]);
  }
  return Object.fromEntries(selectedColumns);
};
const extractSelectedColumnsFromNode = (info, table) => {
  if (!info.selectionSet)
    return {};
  const tableColumns = getTableColumns(table);
  const selectedColumns = [];
  for (const columnSelection of info.selectionSet.selections) {
    if (columnSelection.kind !== Kind.FIELD || !tableColumns[columnSelection.name.value])
      continue;
    selectedColumns.push([columnSelection.name.value, true]);
  }
  return Object.fromEntries(selectedColumns);
};
const extractSelectedColumnsSQLFormat = (info, queryName, table) => {
  const tableSelection = info.operation.selectionSet.selections.find(
    (e) => e.kind === Kind.FIELD && e.name.value === queryName
  );
  const selectedColumns = [];
  if (!tableSelection || !tableSelection.selectionSet)
    throw new GraphQLError("Received empty column selection!");
  for (const columnSelection of tableSelection.selectionSet.selections) {
    if (columnSelection.kind !== Kind.FIELD)
      continue;
    selectedColumns.push([columnSelection.name.value, table[columnSelection.name.value]]);
  }
  return Object.fromEntries(selectedColumns);
};
const innerOrder = new GraphQLInputObjectType({
  name: "InnerOrder",
  fields: {
    direction: {
      type: new GraphQLNonNull(
        new GraphQLEnumType({
          name: "OrderDirection",
          description: "Order by direction",
          values: {
            asc: {
              value: "asc",
              description: "Ascending order"
            },
            desc: {
              value: "desc",
              description: "Descending order"
            }
          }
        })
      )
    },
    priority: { type: new GraphQLNonNull(GraphQLInt), description: "Priority of current field" }
  }
});
const generateColumnFilterValues = (column, tableName, columnName) => {
  const columnGraphQLType = drizzleColumnToGraphQLType(column, true);
  const columnArr = new GraphQLList(new GraphQLNonNull(columnGraphQLType.type));
  const baseFields = {
    eq: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    ne: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    like: { type: GraphQLString },
    notLike: { type: GraphQLString },
    ilike: { type: GraphQLString },
    notIlike: { type: GraphQLString },
    inArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    notInArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    isNull: { type: GraphQLBoolean },
    isNotNull: { type: GraphQLBoolean }
  };
  const type = new GraphQLInputObjectType({
    name: `${pascalize(tableName)}${pascalize(columnName)}Filters`,
    fields: {
      ...baseFields,
      OR: {
        type: new GraphQLList(
          new GraphQLNonNull(
            new GraphQLInputObjectType({
              name: `${pascalize(tableName)}${pascalize(columnName)}filtersOr`,
              fields: {
                ...baseFields
              }
            })
          )
        )
      }
    }
  });
  return type;
};
const filterMap = /* @__PURE__ */ new WeakMap();
const generateTableFilterValuesCached = (table, tableName) => {
  if (filterMap.has(table))
    return filterMap.get(table);
  const columns = getTableColumns(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      {
        type: generateColumnFilterValues(columnDescription, tableName, columnName)
      }
    ])
  );
  filterMap.set(table, remapped);
  return remapped;
};
const fieldMap = /* @__PURE__ */ new WeakMap();
const generateTableSelectTypeFieldsCached = (table) => {
  if (fieldMap.has(table))
    return fieldMap.get(table);
  const columns = getTableColumns(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription)
    ])
  );
  fieldMap.set(table, remapped);
  return remapped;
};
const generateTableTypes = (tableName, table, withReturning, relations) => {
  const relationEntries = relations ? Object.entries(relations) : [];
  const relationFields = Object.fromEntries(
    relationEntries.map(([relName, relValue]) => {
      const relTableFields = Object.fromEntries(
        Object.entries(getTableColumns(relValue.referencedTable)).map(([columnName, columnDescription]) => [
          columnName,
          drizzleColumnToGraphQLType(columnDescription)
        ])
      );
      const type = new GraphQLObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}Relation`,
        fields: relTableFields
      });
      const relationFilterColumns = generateTableFilterValuesCached(relValue.referencedTable, relName);
      const where = new GraphQLInputObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}RelationFilters`,
        fields: {
          ...relationFilterColumns,
          OR: {
            type: new GraphQLList(
              new GraphQLNonNull(
                new GraphQLInputObjectType({
                  name: `${pascalize(tableName)}${pascalize(relName)}RelationFiltersOr`,
                  fields: relationFilterColumns
                })
              )
            )
          }
        }
      });
      if (is(relValue, One))
        return [
          relName,
          {
            type,
            args: {
              where: { type: where }
            }
          }
        ];
      const orderBy = new GraphQLInputObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}RelationOrder`,
        fields: Object.fromEntries(
          Object.entries(relValue.referencedTable).map(([columnName, columnDescription]) => [
            columnName,
            { type: innerOrder }
          ])
        )
      });
      return [
        relName,
        {
          type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(type))),
          args: {
            where: { type: where },
            orderBy: { type: orderBy },
            offset: { type: GraphQLInt },
            limit: { type: GraphQLInt }
          }
        }
      ];
    })
  );
  const tableFields = generateTableSelectTypeFieldsCached(table);
  const columns = getTableColumns(table);
  const columnEntries = Object.entries(columns);
  const insertFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription, false, true)
    ])
  );
  const updateFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription, true)
    ])
  );
  const insertInput = new GraphQLInputObjectType({
    name: `${pascalize(tableName)}InsertInput`,
    fields: insertFields
  });
  const selectSingleOutput = new GraphQLObjectType({
    name: `${pascalize(tableName)}SelectItem`,
    fields: { ...tableFields, ...relationFields }
  });
  const selectArrOutput = new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(selectSingleOutput)));
  const singleTableItemOutput = withReturning ? new GraphQLObjectType({
    name: `${pascalize(tableName)}Item`,
    fields: tableFields
  }) : void 0;
  const arrTableItemOutput = withReturning ? new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(singleTableItemOutput))) : void 0;
  const updateInput = new GraphQLInputObjectType({
    name: `${pascalize(tableName)}UpdateInput`,
    fields: updateFields
  });
  const tableOrder = new GraphQLInputObjectType({
    name: `${pascalize(tableName)}OrderBy`,
    fields: Object.fromEntries(
      columnEntries.map(([columnName, columnDescription]) => [columnName, { type: innerOrder }])
    )
  });
  const filterColumns = generateTableFilterValuesCached(table, tableName);
  const tableFilters = new GraphQLInputObjectType({
    name: `${pascalize(tableName)}Filters`,
    fields: {
      ...filterColumns,
      OR: {
        type: new GraphQLList(
          new GraphQLNonNull(
            new GraphQLInputObjectType({
              name: `${pascalize(tableName)}FiltersOr`,
              fields: filterColumns
            })
          )
        )
      }
    }
  });
  const inputs = {
    insertInput,
    updateInput,
    tableOrder,
    tableFilters
  };
  const outputs = withReturning ? {
    selectSingleOutput,
    selectArrOutput,
    singleTableItemOutput,
    arrTableItemOutput
  } : {
    selectSingleOutput,
    selectArrOutput
  };
  return {
    inputs,
    outputs
  };
};
const extractOrderBy = (table, orderArgs) => {
  const res = [];
  for (const [column, config] of Object.entries(orderArgs).sort(
    (a, b) => (b[1]?.priority ?? 0) - (a[1]?.priority ?? 0)
  )) {
    if (!config)
      continue;
    const { direction } = config;
    res.push(direction === "asc" ? asc(getTableColumns(table)[column]) : desc(getTableColumns(table)[column]));
  }
  return res;
};
const extractFiltersColumn = (column, columnName, operators) => {
  if (!operators.OR?.length)
    delete operators.OR;
  const entries = Object.entries(operators);
  if (operators.OR) {
    if (entries.length > 1)
      throw new GraphQLError(`WHERE ${columnName}: Cannot specify both fields and 'OR' in column operators!`);
    const variants2 = [];
    for (const variant of operators.OR) {
      const extracted = extractFiltersColumn(column, columnName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? or(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [operatorName, operatorValue] of entries) {
    if (operatorValue === null || operatorValue === false)
      continue;
    let operator;
    switch (operatorName) {
      case "eq":
        operator = operator ?? eq;
      case "ne":
        operator = operator ?? ne;
      case "gt":
        operator = operator ?? gt;
      case "gte":
        operator = operator ?? gte;
      case "lt":
        operator = operator ?? lt;
      case "lte":
        operator = operator ?? lte;
        const singleValue = remapFromGraphQLCore(operatorValue, column, columnName);
        variants.push(operator(column, singleValue));
        break;
      case "like":
        operator = operator ?? like;
      case "notLike":
        operator = operator ?? notLike;
      case "ilike":
        operator = operator ?? ilike;
      case "notIlike":
        operator = operator ?? notIlike;
        variants.push(operator(column, operatorValue));
        break;
      case "inArray":
        operator = operator ?? inArray;
      case "notInArray":
        operator = operator ?? notInArray;
        if (!operatorValue.length)
          throw new GraphQLError(
            `WHERE ${columnName}: Unable to use operator ${operatorName} with an empty array!`
          );
        const arrayValue = operatorValue.map((val) => remapFromGraphQLCore(val, column, columnName));
        variants.push(operator(column, arrayValue));
        break;
      case "isNull":
        operator = operator ?? isNull;
      case "isNotNull":
        operator = operator ?? isNotNull;
        variants.push(operator(column));
    }
  }
  return variants.length ? variants.length > 1 ? and(...variants) : variants[0] : void 0;
};
const extractFilters = (table, tableName, filters) => {
  if (!filters.OR?.length)
    delete filters.OR;
  const entries = Object.entries(filters);
  if (!entries.length)
    return;
  if (filters.OR) {
    if (entries.length > 1)
      throw new GraphQLError(`WHERE ${tableName}: Cannot specify both fields and 'OR' in table filters!`);
    const variants2 = [];
    for (const variant of filters.OR) {
      const extracted = extractFilters(table, tableName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? or(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [columnName, operators] of entries) {
    if (operators === null)
      continue;
    const column = getTableColumns(table)[columnName];
    variants.push(extractFiltersColumn(column, columnName, operators));
  }
  return variants.length ? variants.length > 1 ? and(...variants) : variants[0] : void 0;
};
const extractRelationsParams = (relations, tableSelection, typeName, info) => {
  const fields = {};
  const parsedInfo = parseResolveInfo(info, {
    deep: true
  });
  const baseField = parsedInfo ? Object.entries(parsedInfo.fieldsByTypeName).find(([key, value]) => key === typeName)?.[1] ?? {} : {};
  for (const [relName, relValue] of Object.entries(relations)) {
    if (!tableSelection.selectionSet)
      continue;
    const node = tableSelection.selectionSet.selections.find(
      (e) => e.kind === Kind.FIELD && e.name.value === relName
    );
    if (!node)
      continue;
    const refTable = relValue.referencedTable;
    const columns = extractSelectedColumnsFromNode(node, refTable);
    const relationArgs = Object.values(baseField).find((e) => e.name === relName)?.args;
    if (!relationArgs) {
      fields[relName] = {
        columns
      };
      continue;
    }
    const orderBy = relationArgs.orderBy ? extractOrderBy(refTable, relationArgs.orderBy) : void 0;
    const where = relationArgs.where ? extractFilters(refTable, relName, relationArgs?.where) : void 0;
    const offset = relationArgs.offset ?? void 0;
    const limit = relationArgs.limit ?? void 0;
    fields[relName] = {
      columns,
      orderBy,
      where,
      offset,
      limit
    };
  }
  return Object.keys(fields).length ? fields : void 0;
};
export {
  extractFilters,
  extractFiltersColumn,
  extractOrderBy,
  extractRelationsParams,
  extractSelectedColumns,
  extractSelectedColumnsFromNode,
  extractSelectedColumnsSQLFormat,
  generateTableTypes,
  innerOrder
};
//# sourceMappingURL=common.js.map