import { Relations, createTableRelationsHelpers, is } from "drizzle-orm";
import { MySqlTable } from "drizzle-orm/mysql-core";
import {
  GraphQLBoolean,
  GraphQLError,
  GraphQLInt,
  GraphQLList,
  GraphQLNonNull,
  GraphQLObjectType,
  Kind
} from "graphql";
import {
  extractFilters,
  extractOrderBy,
  extractRelationsParams,
  extractSelectedColumnsFromNode,
  generateTableTypes
} from "./common";
import { camelize, pascalize } from "../../caseOps";
import {
  remapFromGraphQLArrayInput,
  remapFromGraphQLSingleInput,
  remapToGraphQLArrayOutput,
  remapToGraphQLSingleOutput
} from "../../dataMappers";
const generateSelectArray = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: GraphQLInt
    },
    limit: {
      type: GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { offset, limit, orderBy, where } = args;
      const tableSelection = info.operation.selectionSet.selections.find(
        (e) => e.kind === Kind.FIELD && e.name.value === queryName
      );
      const query = queryBase.findMany({
        columns: extractSelectedColumnsFromNode(tableSelection, table),
        offset,
        limit,
        orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
        where: where ? extractFilters(table, tableName, where) : void 0,
        with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
      });
      const result = await query;
      return remapToGraphQLArrayOutput(result, relations);
    },
    args: queryArgs
  };
};
const generateSelectSingle = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: GraphQLInt
    },
    limit: {
      type: GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { offset, orderBy, where } = args;
      const tableSelection = info.operation.selectionSet.selections.find(
        (e) => e.kind === Kind.FIELD && e.name.value === queryName
      );
      const columns = extractSelectedColumnsFromNode(tableSelection, table);
      const withFields = relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0;
      const query = queryBase.findFirst({
        columns,
        offset,
        orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
        where: where ? extractFilters(table, tableName, where) : void 0,
        with: withFields
      });
      const result = await query;
      if (!result)
        return void 0;
      return remapToGraphQLSingleOutput(result, relations);
    },
    args: queryArgs
  };
};
const generateInsertArray = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}`;
  const queryArgs = {
    values: {
      type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const input = remapFromGraphQLArrayInput(args.values, table);
      if (!input.length)
        throw new GraphQLError("No values were provided!");
      try {
        await db.insert(table).values(input);
        return { isSuccess: true };
      } catch (e) {
        return { isSuccess: false };
      }
    },
    args: queryArgs
  };
};
const generateInsertSingle = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const input = remapFromGraphQLSingleInput(args.values, table);
      try {
        await db.insert(table).values(input);
        return { isSuccess: true };
      } catch (e) {
        return { isSuccess: false };
      }
    },
    args: queryArgs
  };
};
const generateUpdate = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${pascalize(tableName)}`;
  const queryArgs = {
    set: {
      type: new GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { where, set } = args;
      const input = remapFromGraphQLSingleInput(set, table);
      if (!Object.keys(input).length)
        throw new GraphQLError("Unable to update with no values specified!");
      let query = db.update(table).set(input);
      if (where) {
        const filters = extractFilters(table, tableName, where);
        query = query.where(filters);
      }
      try {
        await query;
        return { isSuccess: true };
      } catch (e) {
        return { isSuccess: false };
      }
    },
    args: queryArgs
  };
};
const generateDelete = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${tableName}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { where } = args;
      let query = db.delete(table);
      if (where) {
        const filters = extractFilters(table, tableName, where);
        query = query.where(filters);
      }
      try {
        await query;
        return { isSuccess: true };
      } catch (e) {
        return { isSuccess: false };
      }
    },
    args: queryArgs
  };
};
const generateSchemaData = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => is(value, MySqlTable)));
  if (!tables || !Object.keys(tables).length)
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => is(value, Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config(createTableRelationsHelpers(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      generateTableTypes(tableName, table, false, relations[tableName])
    ])
  );
  const mutationReturnType = new GraphQLObjectType({
    name: `MutationReturn`,
    fields: {
      isSuccess: {
        type: new GraphQLNonNull(GraphQLBoolean)
      }
    }
  });
  const inputs = {};
  const outputs = {
    MutationReturn: mutationReturnType
  };
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate(
      db,
      tableName,
      schema[tableName],
      updateInput,
      tableFilters
    );
    const deleteGenerated = generateDelete(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: mutationReturnType,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: mutationReturnType,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: mutationReturnType,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: mutationReturnType,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};
export {
  generateSchemaData
};
//# sourceMappingURL=mysql.js.map