import { getTableColumns } from "drizzle-orm";
import { GraphQLError } from "graphql";
const remapToGraphQLCore = (key, value, relations) => {
  if (value instanceof Date)
    return value.toISOString();
  if (value instanceof Buffer)
    return Array.from(value);
  if (typeof value === "bigint")
    return value.toString();
  if (Array.isArray(value)) {
    if (relations?.[key])
      return remapToGraphQLArrayOutput(value);
    return value.map((arrVal) => remapToGraphQLCore(key, arrVal));
  }
  if (typeof value === "object") {
    if (relations?.[key])
      return remapToGraphQLSingleOutput(value);
    return JSON.stringify(value);
  }
  return value;
};
const remapToGraphQLSingleOutput = (queryOutput, relations) => {
  for (const [key, value] of Object.entries(queryOutput)) {
    if (value === void 0 || value === null) {
      delete queryOutput[key];
    } else {
      queryOutput[key] = remapToGraphQLCore(key, value, relations);
    }
  }
  return queryOutput;
};
const remapToGraphQLArrayOutput = (queryOutput, relations) => {
  for (const entry of queryOutput)
    remapToGraphQLSingleOutput(entry, relations);
  return queryOutput;
};
const remapFromGraphQLCore = (value, column, columnName) => {
  switch (column.dataType) {
    case "date": {
      const formatted = new Date(value);
      if (Number.isNaN(formatted.getTime()))
        throw new GraphQLError(`Field '${columnName}' is not a valid date!`);
      return formatted;
    }
    case "buffer": {
      if (!Array.isArray(value)) {
        throw new GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return Buffer.from(value);
    }
    case "json": {
      try {
        return JSON.stringify(value);
      } catch (e) {
        throw new GraphQLError(
          `Invalid JSON in field '${columnName}':
${e instanceof Error ? e.message : "Unknown error"}`
        );
      }
    }
    case "array": {
      if (!Array.isArray(value)) {
        throw new GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return value;
    }
    case "bigint": {
      try {
        return BigInt(value);
      } catch (error) {
        throw new GraphQLError(`Field '${columnName}' is not a BigInt!`);
      }
    }
    default: {
      return value;
    }
  }
};
const remapFromGraphQLSingleInput = (queryInput, table) => {
  for (const [key, value] of Object.entries(queryInput)) {
    if (value === void 0) {
      delete queryInput[key];
    } else {
      const column = getTableColumns(table)[key];
      if (!column)
        throw new GraphQLError(`Unknown column: ${key}`);
      if (value === null && column.notNull) {
        delete queryInput[key];
        continue;
      }
      queryInput[key] = remapFromGraphQLCore(value, column, key);
    }
  }
  return queryInput;
};
const remapFromGraphQLArrayInput = (queryInput, table) => {
  for (const entry of queryInput)
    remapFromGraphQLSingleInput(entry, table);
  return queryInput;
};
export {
  remapFromGraphQLArrayInput,
  remapFromGraphQLCore,
  remapFromGraphQLSingleInput,
  remapToGraphQLArrayOutput,
  remapToGraphQLCore,
  remapToGraphQLSingleOutput
};
//# sourceMappingURL=index.js.map